use axum::{
    http::{StatusCode, header},
    response::{IntoResponse, Response},
};
use snafu::{ResultExt, Snafu};
use time::OffsetDateTime;

use crate::{
    APP_NAME,
    api::{
        OPDS_TAG,
        error::{ErrorResponse, HttpStatus},
    },
    http_error,
    opds::{
        content::Content, entry::Entry, error::AsXmlError, feed::Feed, link::Link,
        media_type::MediaType, relation::Relation,
    },
};

#[derive(Debug, Snafu)]
/// Errors that can occur when rendering the root OPDS feed.
pub enum OdpsFeedError {
    /// A failure to create the OPDS feed.
    #[snafu(display("Failed to create opds feed."))]
    Feed { source: AsXmlError },
}
impl HttpStatus for OdpsFeedError {
    fn status_code(&self) -> StatusCode {
        match self {
            OdpsFeedError::Feed { source: _ } => StatusCode::INTERNAL_SERVER_ERROR,
        }
    }
}
http_error!(OdpsFeedError);

/// Render all books as OPDS entries embedded in a feed.
#[utoipa::path(
    get,
    path = "/",
    tag = OPDS_TAG,
    responses(
        (status = OK, content_type = "application/atom+xml"),
        (status = 500, description = "Server failure.", body = ErrorResponse)
    )
)]
pub async fn handler() -> Result<Response, OdpsFeedError> {
    let now = OffsetDateTime::now_utc();

    let self_link = Link {
        href: "/opds".to_string(),
        media_type: MediaType::Navigation,
        rel: Relation::Myself,
        title: None,
        count: None,
    };
    let books_entry = Entry {
        title: "Books".to_string(),
        id: format!("{APP_NAME}:books"),
        updated: now,
        content: Some(Content {
            media_type: MediaType::Text,
            content: "Index of all books".to_string(),
        }),
        author: None,
        links: vec![Link {
            href: "/opds/books".to_string(),
            media_type: MediaType::Navigation,
            rel: Relation::Subsection,
            title: None,
            count: None,
        }],
    };

    let authors_entry = Entry {
        title: "Authors".to_string(),
        id: format!("{APP_NAME}:authors"),
        updated: now,
        content: Some(Content {
            media_type: MediaType::Text,
            content: "Index of all authors".to_string(),
        }),
        author: None,
        links: vec![Link {
            href: "/opds/authors".to_string(),
            media_type: MediaType::Navigation,
            rel: Relation::Subsection,
            title: None,
            count: None,
        }],
    };

    let series_entry = Entry {
        title: "Series".to_string(),
        id: format!("{APP_NAME}:series"),
        updated: now,
        content: Some(Content {
            media_type: MediaType::Text,
            content: "Index of all series".to_string(),
        }),
        author: None,
        links: vec![Link {
            href: "/opds/series".to_string(),
            media_type: MediaType::Navigation,
            rel: Relation::Subsection,
            title: None,
            count: None,
        }],
    };

    let recents_entry = Entry {
        title: "Recent Additions".to_string(),
        id: format!("{APP_NAME}:recentbooks"),
        updated: now,
        content: Some(Content {
            media_type: MediaType::Text,
            content: "Recently added books".to_string(),
        }),
        author: None,
        links: vec![Link {
            href: "/opds/recent".to_string(),
            media_type: MediaType::Navigation,
            rel: Relation::Subsection,
            title: None,
            count: None,
        }],
    };

    let feed = Feed::create(
        now,
        &format!("{APP_NAME}:catalog"),
        "Little Hesinde",
        self_link,
        vec![],
        vec![authors_entry, series_entry, books_entry, recents_entry],
    );
    let xml = feed.as_xml().context(FeedSnafu)?;

    Ok(([(header::CONTENT_TYPE, "application/atom+xml")], xml).into_response())
}
